# coding: utf-8
u"""
Модели приложения
"""
#------------------------------------------------------------------------------
# Imports
#------------------------------------------------------------------------------

# Stdlib
from __future__ import absolute_import
import json

# 3rdparty
from django.db import models
import six

#------------------------------------------------------------------------------
# Models
#------------------------------------------------------------------------------

class Person(models.Model):
    u"""
    Физическое лицо (человек)
    """

    GENDERS = {
        0: u'муж.',
        1: u'жен',
    }
    GENDERS_LIST = list(GENDERS.items())

    fname = models.CharField(
        u'Имя', max_length=255)
    sname = models.CharField(
        u'Фамилия', max_length=255)
    mname = models.CharField(
        u'Отчество', max_length=255)
    birthday = models.DateField(
        u'День рождения', null=True)
    gender = models.SmallIntegerField(
        u'Пол',
        choices=six.iteritems(GENDERS),
        default=0)
    

    # Properties
    # -----------------------------------------------------------------
    
    @property
    def fullname(self):
        return u'{.fname} {.sname} {.mname}'.format(self).rstrip()

    @property
    def shortname(self):
        return u'{fname} .{sname} .{mname}'.format(
            fname=self.fname,
            sname=(self.sname or ' ')[0],
            mname=(self.mname or ' ')[0]
        ).rstrip(' .')


    # Meta
    # -----------------------------------------------------------------

    class Meta:
        verbose_name = u'Физическое лицо'
        verbose_name_plural = u'Физические лица'



# coding: utf-8

#------------------------------------------------------------------------------
# Metadata
#------------------------------------------------------------------------------

__author__ = 'Andrey Torsunov'
__contact__ = 'torsunov@bars-open.com'
__docformat__ = 'restructuredtext'


#------------------------------------------------------------------------------
# Exception classes
#------------------------------------------------------------------------------

class ProviderException(Exception):
    u""" Ошибка провайдера данных.

    Базовое исключение для всех ошибок провайдера данных.

    """
    def __init__(self, source=None, field=None, key=None):
        self.source = source 
        self.field = field
        self.key = key
        self._message_tpl = '@source={source} @field={field} @key={key}'
        super(ProviderException, self).__init__(self._build_message())

    def _build_message(self):
        message = ''
        if self.source and self.field and self.key:
            try:
                source_name = self.source.__name__
            except AttributeError:
                source_name = self.source.__class__.__name__
            message = self._message_tpl.format(
                source=source_name,
                field=self.field,
                key=self.key)
        return message


class ObjectDoesNotExists(ProviderException):
    u""" Запись не существует.

    Возникает если не удалось найти запись по фильтру.

    """
    pass


class MultipleObjects(ProviderException):
    u""" Множественные данные вместо единичной записи.

    Возникает если по фильтру было получено больше одной записи.

    """
    pass



# coding: utf-8

#------------------------------------------------------------------------------
# Imports
#------------------------------------------------------------------------------
from __future__ import absolute_import

import decimal
import datetime
import json
import six

from six import string_types, text_type

#------------------------------------------------------------------------------
# Metadata
#------------------------------------------------------------------------------

__author__ = 'Andrey Torsunov'
__contact__ = 'torsunov@bars-open.com'
__docformat__ = 'restructuredtext'


#------------------------------------------------------------------------------
# Typecast implementation
#------------------------------------------------------------------------------

DATETIME_FORMATS = (
    '%Y-%m-%dT%H:%M:%S',
    '%Y-%m-%d %H:%M:%S',
    '%d.%m.%Y %H:%M:%S',
    '%Y-%m-%dT%H:%M',
    '%Y-%m-%d %H:%M',
    '%d.%m.%Y %H:%M',
)


DATE_FORMATS = (
    '%Y-%m-%d',
    '%d.%m.%Y',
    '%m/%d/%Y',
)


def cast_to_datetime(raw, formats=DATETIME_FORMATS):
    return _date_parser(raw, formats)


def cast_to_date(raw, formats=DATE_FORMATS+DATETIME_FORMATS):
    u"""
        datetime - это тоже date. Поэтому если мы передали строку в более
        точном формате - то не валимся, а обрезаем до даты.
    """
    dt = _date_parser(raw, formats)
    return datetime.datetime(dt.year, dt.month, dt.day)


def cast_to_time(raw):
    formats = (
        '%H:%M:%S',
        '%H:%M',
    )
    return _date_parser(raw, formats)


def _date_parser(raw, formats, fallback_parser=None):
    for fmt in formats:
        try:
            converted = datetime.datetime.strptime(raw, fmt)
        except ValueError:
            pass
        else:
            return converted
    else:
        raise ValueError((
            u"Time data '{data!r}' does not match any of known formats:"
            u"\n{formats}"
        ).format(
            data=raw,
            formats='\n'.join(formats))
        )


def typecast(raw, type_, custom_cast=None):
    u""" Приведение данных к определенному типу.

    :param raw: сырые данные
    :param type_: тип приведения
    :param dict custom_cast: позволяется переопредилить порядок приведения,
        в качестве ключа выступает класс типа (:class:`int`, :class:`str`,
        :class:`bool` и т.д), в качестве значение callable с одним аргументом

    :return: значение приведенное к *type_*

    :raise: ValueError, TypeError

    """
    raw = six.text_type(raw)
    if type_ != six.string_types:
        raw = raw.strip()
    custom_cast = custom_cast or {}
    cast_map = {**CAST_MAP, **custom_cast}
    cast = cast_map.get(type_)
    if cast is None:
        raise ValueError("Type converter to '%s' is not defined" % repr(type_))
    return cast(raw)


def mute_typecast(raw, _type, custom_cast=None, default=None):
    u""" Тихое приведение данных к определенному типу.

    Имеет тот же функционал, что и :func:`typecast`, но в случае
    возникновения исключения возвращает *default*.

    """
    try:
        converted = typecast(raw, _type, custom_cast)
    finally:
        return locals().get('converted', default)


CAST_MAP = {
    int: int,
    float: float,
    bool: lambda x: x.lower() in ('true', '1', 'on'),
    decimal.Decimal: lambda x: decimal.Decimal(x),
    datetime.datetime: cast_to_datetime,
    datetime.date: cast_to_date,
    datetime.time: cast_to_time,
    object: json.loads
}
for t in string_types:
    CAST_MAP[t] = text_type



# coding: utf-8
from __future__ import absolute_import

import six
import datetime
import decimal

from json import JSONEncoder


class JsonSerializableEncoder(JSONEncoder):
    def __init__(self, *args, **kwargs):
        kwargs.pop('dict_list', None)
        super(JsonSerializableEncoder, self).__init__(*args, **kwargs)

    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            result = '%02d.%02d.%04d %02d:%02d:%02d' % (
                obj.day, obj.month, obj.year, obj.hour, obj.minute, obj.second)
        elif isinstance(obj, datetime.date):
            result = '%02d.%02d.%04d' % (obj.day, obj.month, obj.year)
        elif isinstance(obj, datetime.time):
            result = obj.strftime('%H:%M')
        elif isinstance(obj, decimal.Decimal):
            result = str(obj)
        else:
            result = {}

            for k, v in six.iteritems(obj.__dict__):
                if not k.startswith('_'):
                    result[k] = v

            cls = obj.__class__

            for attname in dir(cls):
                if not attname.startswith('_'):
                    try:
                        cls_attr = getattr(cls, attname)
                    except Exception:
                        continue
                    if getattr(cls_attr, 'json_encode', False):
                        value = getattr(obj, attname)
                        if callable(value):
                            result[attname] = value()
                        else:
                            result[attname] = value
        return result



# coding: utf-8

#------------------------------------------------------------------------------
# Imports
#------------------------------------------------------------------------------

# 3rdparty
from __future__ import absolute_import

from django.dispatch import Signal

#------------------------------------------------------------------------------
# Metadata
#------------------------------------------------------------------------------

__author__ = 'Andrey Torsunov'
__contact__ = 'torsunov@bars-open.com'
__docformat__ = 'restructuredtext'

#------------------------------------------------------------------------------
# Signals
#------------------------------------------------------------------------------


# Recordpack
# ---------------------------------------------------------------------

#: Сигнал создания журнального окна, вызывается в:
#: :meth:`~recordpack.recordpack.BaseRecordListPack.request_list_window`
base_record_list_pack_after_get_list_window = Signal(
    providing_args=['instance', 'request', 'context', 'window'])

#: Сигнал создания окна выбора записи, вызывается в:
#: :meth:`~recordpack.recordpack.BaseRecordListPack.request_select_window`
base_record_list_pack_after_get_select_window = Signal(
    providing_args=['instance', 'request', 'context', 'window'])

#: Сигнал создания окна редактирования, вызывается в:
#: :meth:`~recordpack.recordpack.BaseRecordPack.request_edit_window`
base_record_pack_after_get_edit_window = Signal(
    providing_args=['instance', 'request', 'context', 'record', 'window',
                    'is_new'])

#: Сигнал перед извлечением записей, вызывается в:
#: :meth:`~recordpack.recordpack.BaseRecordPack.request_rows`
base_record_pack_before_get_rows = Signal(
    providing_args=['instance', 'request', 'context', 'query'])

#: Сигнал после извлечения записей, вызывается в:
#: :meth:`~recordpack.recordpack.BaseRecordPack.request_rows`
base_record_pack_after_get_rows = Signal(
    providing_args=['instance', 'request', 'context', 'query', 'rows'])


# Providers
# ---------------------------------------------------------------------

#: Сигнал определения менеджера, вызывается в:
#: :meth:`~recordpack.provider.DjangoModelProvider.get_manager`
provider_get_manager = Signal(providing_args=['query_object', 'manager'])
